@@ -1,7 +1,5 @@
-import typing
 import pickle
 import pytest
-import sys
 import numpy as np
 
 from arraymap import AutoMap
@@ -92,6 +90,23 @@ def test_fam_constructor_array_int_d():
         assert k in fam
 
 
+def test_fam_constructor_array_int_e():
+    # https://github.com/static-frame/arraymap/issues/12
+    a1 = np.array((0, 0, 1, 1, 2, 2), dtype=int)
+    a2 = a1[[0, 2, 4]]
+    a2.flags.writeable = False
+    fam1 = FrozenAutoMap(a2)
+    assert list(fam1) == [0, 1, 2]
+
+    d1 = {i: int(i) for i in a2}
+    fam2 = FrozenAutoMap(d1)
+    assert list(fam2) == [0, 1, 2]
+
+    d2 = {0: 0, 3: 1}
+    fam3 = FrozenAutoMap(d2)
+    assert list(fam3) == [0, 3]
+
+
 # ------------------------------------------------------------------------------
 
 
@@ -106,11 +121,36 @@ def test_fam_constructor_array_float_a():
 
 
 def test_fam_constructor_array_dt64_a():
-    a1 = np.array(("2022-01", "2023-05"), dtype=np.datetime64)
+    a1 = np.array(("1970-01", "2023-05"), dtype=np.datetime64)
     a1.flags.writeable = False
     fam = FrozenAutoMap(a1)
+
     assert fam[np.datetime64("2023-05")] == 1
-    # assert np.datetime64('2022-05') in a1
+    assert fam[np.datetime64("1970-01")] == 0
+
+    with pytest.raises(KeyError):
+        fam[np.datetime64("nat")]
+
+    with pytest.raises(KeyError):
+        fam[np.datetime64("1970")]
+
+
+def test_fam_constructor_array_dt64_b():
+    a1 = np.array(("1542", "nat"), dtype=np.datetime64)
+    a1.flags.writeable = False
+    fam = FrozenAutoMap(a1)
+    assert fam[np.datetime64("nat")] == 1
+    assert fam[np.datetime64("nat", "D")] == 1
+    assert fam[np.datetime64("nat", "ns")] == 1
+    assert fam[np.datetime64("1542")] == 0
+
+
+def test_fam_constructor_array_dt64_c():
+    a1 = np.array(("nat", "nat"), dtype=np.datetime64)
+    a1.flags.writeable = False
+    fam = FrozenAutoMap(a1)
+    # when we get "generic" dt64 units, we load scalars in a list, and can thus support multiple NaNs
+    assert len(fam) == 2
 
 
 # ------------------------------------------------------------------------------
@@ -217,6 +257,20 @@ def test_fam_copy_array_bytes_a():
 # ------------------------------------------------------------------------------
 
 
+def test_fam_array_bytes_get_a():
+    a1 = np.array((b"", b"  ", b"   ", b"    "))
+    a1.flags.writeable = False
+    fam = FrozenAutoMap(a1)
+
+    assert fam.get(b"") == 0
+    assert fam.get(b" ") == None
+    assert fam.get(b"   ") == 2
+    assert fam.get(b"    ") == 3
+
+
+# ------------------------------------------------------------------------------
+
+
 def test_fam_array_len_a():
     a1 = np.array((10, 20, 30, 40), dtype=np.int64)
     a1.flags.writeable = False
@@ -358,6 +412,14 @@ def test_fam_array_int_get_d():
     assert 4 in fam
 
 
+def test_fam_array_int_get_e():
+    a1 = np.array((1,), dtype=np.int16)
+    a1.flags.writeable = False
+    fam = FrozenAutoMap(a1)
+    assert a1[0] in fam
+    assert 1 in fam
+
+
 # ------------------------------------------------------------------------------
 
 
@@ -552,6 +614,17 @@ def test_fam_array_unicode_get_a():
     assert fam.get("cccc") is None
 
 
+def test_fam_array_unicode_get_b():
+    a1 = np.array(("", "  ", "   ", "    "))
+    a1.flags.writeable = False
+    fam = FrozenAutoMap(a1)
+
+    assert fam.get("") == 0
+    assert fam.get(" ") == None
+    assert fam.get("   ") == 2
+    assert fam.get("    ") == 3
+
+
 # ------------------------------------------------------------------------------
 
 
@@ -647,9 +720,246 @@ def test_fam_array_pickle_a():
     a1 = np.array(("a", "b", "c", "d"))
     a1.flags.writeable = False
     fam1 = FrozenAutoMap(a1)
-
     fam2 = pickle.loads(pickle.dumps(fam1))
+    assert list(fam1.values()) == list(fam2.values())
 
-    # import ipdb
 
-    # ipdb.set_trace()
+# ------------------------------------------------------------------------------
+
+
+def test_fam_array_get_all_a():
+    a1 = np.array((1, 100, 300, 4000))
+    a1.flags.writeable = False
+    fam = FrozenAutoMap(a1)
+
+    with pytest.raises(TypeError):
+        fam.get_all((3, 3))
+
+    with pytest.raises(TypeError):
+        fam.get_all("a")
+
+    with pytest.raises(TypeError):
+        fam.get_all(None)
+
+
+def test_fam_array_get_all_b():
+    a1 = np.array((1, 100, 300, 4000))
+    a1.flags.writeable = False
+    fam = FrozenAutoMap(a1)
+    post1 = fam.get_all([300, 100])
+    assert post1.tolist() == [2, 1]
+    x = [y for y in post1]
+    del x
+    del post1
+    post2 = fam.get_all([4000, 4000, 4000])
+    assert post2.tolist() == [3, 3, 3]
+    x = [y for y in post2]
+    del x
+
+
+def test_fam_array_get_all_c():
+    a1 = np.array(("a", "bb", "ccc"))
+    a1.flags.writeable = False
+    fam = FrozenAutoMap(a1)
+    with pytest.raises(KeyError):
+        fam.get_all(["bb", "c"])
+
+
+def test_fam_array_get_all_d1():
+    a1 = np.array(("a", "bb", "ccc"))
+    a1.flags.writeable = False
+    fam = FrozenAutoMap(a1)
+    post1 = fam.get_all(np.array(("bb", "a", "ccc", "a", "bb")))
+    assert post1.tolist() == [1, 0, 2, 0, 1]
+    assert post1.flags.writeable == False
+
+
+def test_fam_array_get_all_d2():
+    a1 = np.array(("a", "bb", "ccc"))
+    a1.flags.writeable = False
+    fam = FrozenAutoMap(a1)
+    with pytest.raises(KeyError):
+        fam.get_all(np.array(("bb", "a", "ccc", "aa")))
+
+
+def test_fam_array_get_all_e():
+    a1 = np.array((2,), dtype=np.uint64)
+    a1.flags.writeable = False
+    fam = FrozenAutoMap(a1)
+    assert fam.get_all([2]) == [0]
+    assert fam.get_all(a1) == [0]
+
+
+def test_fam_array_get_all_f1():
+    a1 = np.array(("a", "bb", "ccc", "dd"))
+    a1.flags.writeable = False
+    fam = FrozenAutoMap(a1)
+
+    post = fam.get_all(np.array(["ccc", "dd", "bb", "bb"]))
+    assert post.tolist() == [2, 3, 1, 1]
+
+
+def test_fam_array_get_all_f2():
+    a1 = np.array(("a", "bb", "ccc", "dd"))
+    a1.flags.writeable = False
+    fam = FrozenAutoMap(a1)
+
+    with pytest.raises(KeyError):
+        fam.get_all(np.array(["bb", "c"]))
+
+
+def test_fam_array_get_all_g1():
+    a1 = np.array((b"a", b"bb", b"ccc", b"dd"))
+    a1.flags.writeable = False
+    fam = FrozenAutoMap(a1)
+
+    post = fam.get_all(np.array([b"ccc", b"dd", b"bb", b"bb"]))
+    assert post.tolist() == [2, 3, 1, 1]
+
+
+def test_fam_array_get_all_g2():
+    a1 = np.array((b"a", b"bb", b"ccc", b"dd"))
+    a1.flags.writeable = False
+    fam = FrozenAutoMap(a1)
+
+    with pytest.raises(KeyError):
+        fam.get_all(np.array([b"dd", b"x"]))
+
+
+def test_fam_array_get_all_h():
+    a1 = np.array((b"a", b""))
+    a1.flags.writeable = False
+    fam = FrozenAutoMap(a1)
+
+    post = fam.get_all(np.array([b"", b"", b"a"]))
+    assert post.tolist() == [1, 1, 0]
+
+
+def test_fam_array_get_all_i():
+    a1 = np.array((b"foo", b"bar"))
+    a1.flags.writeable = False
+    fam = FrozenAutoMap(a1)
+    with pytest.raises(KeyError):
+        _ = fam.get_all(np.array([b"fo", b"ba"]))
+
+    with pytest.raises(KeyError):
+        _ = fam.get_all(np.array([b"", b""]))
+
+
+def test_fam_array_get_all_j():
+    a1 = np.array(("aaaaa", "bb", "ccc", "dd"))
+    a1.flags.writeable = False
+    fam = FrozenAutoMap(a1)
+
+    with pytest.raises(KeyError):
+        _ = fam.get_all(np.array(["a", "b"]))
+
+    assert fam.get_all(np.array(("bb", "dd", "bb", "dd"))).tolist() == [1, 3, 1, 3]
+
+
+def test_fam_array_get_all_k1():
+    a1 = np.array(("2023-01-05", "1854-05-02"), np.datetime64)
+    a1.flags.writeable = False
+    fam = FrozenAutoMap(a1)
+
+    post = fam.get_all(
+        np.array(["1854-05-02", "2023-01-05", "2023-01-05"], np.datetime64)
+    )
+    assert post.tolist() == [1, 0, 0]
+
+
+def test_fam_array_get_all_k2():
+    a1 = np.array(("2023-01-05", "1854-05-02"), np.datetime64)
+    a1.flags.writeable = False
+    fam = FrozenAutoMap(a1)
+
+    with pytest.raises(KeyError):
+        post = fam.get_all(
+            np.array(["1854-05-02", "2023-01-05", "2020-01-05"], np.datetime64)
+        )
+
+
+def test_fam_array_get_all_l():
+    a1 = np.array(("2023-01-05", "1854-05-02", "1988-01-01"), np.datetime64)
+    a1.flags.writeable = False
+    fam = FrozenAutoMap(a1)
+    with pytest.raises(KeyError):
+        _ = fam.get_all(np.array(["2022-01", "2023-01", "1988-01"], np.datetime64))
+
+
+# -------------------------------------------------------------------------------
+
+
+def test_fam_array_get_any_a1():
+    a1 = np.array(("a", "bb", "ccc"))
+    a1.flags.writeable = False
+    fam = FrozenAutoMap(a1)
+    post1 = fam.get_any(["bbb", "ccc", "a", "bbb"])
+    assert post1 == [2, 0]
+
+    post2 = fam.get_any(["bbb", "bbb"])
+    assert post2 == []
+
+
+def test_fam_array_get_any_a2():
+    a1 = np.array(("a", "bb", "ccc"))
+    a1.flags.writeable = False
+    fam = FrozenAutoMap(a1)
+    post1 = fam.get_any(np.array(("bbb", "a", "ccc", "aa", "bbb")))
+    assert post1 == [0, 2]
+
+
+def test_fam_array_get_any_a3():
+    a1 = np.array(("a", "bb", "ccc"))
+    a1.flags.writeable = False
+    fam = FrozenAutoMap(a1)
+    post1 = fam.get_any(np.array(["bbb", "ccc", "a", "bbb"]))
+    assert post1 == [2, 0]
+
+    post2 = fam.get_any(np.array(["bbb", "bbb"]))
+    assert post2 == []
+
+
+def test_fam_array_get_any_b():
+    a1 = np.array([4294967295], dtype=np.uint32)
+    a1.flags.writeable = False
+    a1_list = list(a1)
+    fam = FrozenAutoMap(a1)
+    assert a1[0] in fam
+    assert 4294967295 in fam
+
+    post1 = fam.get_any(a1_list)
+    assert post1 == list(fam.values())
+
+
+def test_fam_array_get_any_c1():
+    a1 = np.array(("2023-01-05", "1854-05-02"), np.datetime64)
+    a1.flags.writeable = False
+    fam = FrozenAutoMap(a1)
+
+    post = fam.get_any(
+        np.array(
+            ["1854-05-02", "nat", "1854-05-02", "2023-01-05", "nat"], np.datetime64
+        )
+    )
+    assert post == [1, 1, 0]
+
+
+def test_fam_array_get_any_c2():
+    a1 = np.array(("2023-01-05", "1854-05-02"), np.datetime64)
+    a1.flags.writeable = False
+    fam = FrozenAutoMap(a1)
+
+    post = fam.get_any(
+        np.array(["1854-05-02", "2023-01-05", "2020-01-05"], np.datetime64)
+    )
+    assert post == [1, 0]
+
+
+def test_fam_array_get_any_d():
+    a1 = np.array(("2023-01-05", "1854-05-02", "1988-01-01"), np.datetime64)
+    a1.flags.writeable = False
+    fam = FrozenAutoMap(a1)
+
+    post = fam.get_any(np.array(["2022-01", "2023-01", "1988-01"], np.datetime64))
+    assert post == []
